@@ -23,12 +23,12 @@ class TestDecimal128DataType < Test::Unit::TestCase
 
   def test_name
     data_type = Arrow::Decimal128DataType.new(2, 0)
-    assert_equal("decimal", data_type.name)
+    assert_equal("decimal128", data_type.name)
   end
 
   def test_to_s
     data_type = Arrow::Decimal128DataType.new(2, 0)
-    assert_equal("decimal(2, 0)", data_type.to_s)
+    assert_equal("decimal128(2, 0)", data_type.to_s)
   end
 
   def test_precision
@@ -40,4 +40,15 @@ class TestDecimal128DataType < Test::Unit::TestCase
     data_type = Arrow::Decimal128DataType.new(8, 2)
     assert_equal(2, data_type.scale)
   end
+
+  def test_decimal_data_type_new
+    assert_equal(Arrow::Decimal128DataType.new(19, 2),
+                 Arrow::DecimalDataType.new(19, 2))
+  end
+
+  def test_invalid_precision
+    assert_raise(Arrow::Error::Invalid) do
+      Arrow::Decimal128DataType.new(39, 1)
+    end
+  end
 end
