@@ -167,7 +167,7 @@ export ARROW_HOME=/tmp/arrow-dist
 export CMAKE_PREFIX_PATH=/tmp/arrow-dist
 
 pushd /arrow/python
-python setup.py bdist_wheel
+python -m build --sdist --wheel . --no-isolation
 
 echo "=== Strip symbols from wheel ==="
 mkdir -p dist/temp-fix-wheel
