@@ -66,7 +66,8 @@
 #'
 #' open_dataset(tf, format = format)
 #' @export
-FileFormat <- R6Class("FileFormat",
+FileFormat <- R6Class(
+  "FileFormat",
   inherit = ArrowObject,
   active = list(
     # @description
@@ -84,7 +85,8 @@ FileFormat$create <- function(format, schema = NULL, partitioning = NULL, ...) {
     CsvFileFormat$create(delimiter = "\t", schema = schema, partitioning = partitioning, ...)
   } else if (format == "parquet") {
     ParquetFileFormat$create(...)
-  } else if (format %in% c("ipc", "arrow", "feather")) { # These are aliases for the same thing
+  } else if (format %in% c("ipc", "arrow", "feather")) {
+    # These are aliases for the same thing
     dataset___IpcFileFormat__Make()
   } else if (format == "json") {
     JsonFileFormat$create(...)
@@ -105,8 +107,7 @@ as.character.FileFormat <- function(x, ...) {
 #' @rdname FileFormat
 #' @export
 ParquetFileFormat <- R6Class("ParquetFileFormat", inherit = FileFormat)
-ParquetFileFormat$create <- function(...,
-                                     dict_columns = character(0)) {
+ParquetFileFormat$create <- function(..., dict_columns = character(0)) {
   options <- ParquetFragmentScanOptions$create(...)
   dataset___ParquetFileFormat__Make(options, dict_columns)
 }
@@ -338,7 +339,8 @@ check_ambiguous_options <- function(passed_opts, opts1, opts2) {
   is_ambig_opt <- is.na(pmatch(passed_opts, c(opts1, opts2)))
   ambig_opts <- passed_opts[is_ambig_opt]
   if (length(ambig_opts)) {
-    stop("Ambiguous ",
+    stop(
+      "Ambiguous ",
       ngettext(length(ambig_opts), "option", "options"),
       ": ",
       oxford_paste(ambig_opts),
@@ -425,9 +427,7 @@ csv_file_format_parse_opts <- function(...) {
     # Catch cases when the user specifies a mix of Arrow C++ options and
     # readr-style options
     if (!all(is_readr_opt)) {
-      stop("Use either Arrow parse options or readr parse options, not both",
-        call. = FALSE
-      )
+      stop("Use either Arrow parse options or readr parse options, not both", call. = FALSE)
     }
     do.call(readr_to_csv_parse_options, opts) # all options have readr-style names
   } else {
@@ -571,7 +571,8 @@ csv_file_format_read_opts <- function(schema = NULL, partitioning = NULL, ...) {
 #' @rdname FragmentScanOptions
 #' @name FragmentScanOptions
 #' @export
-FragmentScanOptions <- R6Class("FragmentScanOptions",
+FragmentScanOptions <- R6Class(
+  "FragmentScanOptions",
   inherit = ArrowObject,
   active = list(
     # @description
@@ -601,9 +602,11 @@ as.character.FragmentScanOptions <- function(x, ...) {
 #' @rdname FragmentScanOptions
 #' @export
 CsvFragmentScanOptions <- R6Class("CsvFragmentScanOptions", inherit = FragmentScanOptions)
-CsvFragmentScanOptions$create <- function(...,
-                                          convert_opts = csv_file_format_convert_opts(...),
-                                          read_opts = csv_file_format_read_opts(...)) {
+CsvFragmentScanOptions$create <- function(
+  ...,
+  convert_opts = csv_file_format_convert_opts(...),
+  read_opts = csv_file_format_read_opts(...)
+) {
   dataset___CsvFragmentScanOptions__Make(convert_opts, read_opts)
 }
 
@@ -612,13 +615,18 @@ CsvFragmentScanOptions$create <- function(...,
 #' @rdname FragmentScanOptions
 #' @export
 ParquetFragmentScanOptions <- R6Class("ParquetFragmentScanOptions", inherit = FragmentScanOptions)
-ParquetFragmentScanOptions$create <- function(use_buffered_stream = FALSE,
-                                              buffer_size = 8196,
-                                              pre_buffer = TRUE,
-                                              thrift_string_size_limit = 100000000,
-                                              thrift_container_size_limit = 1000000) {
+ParquetFragmentScanOptions$create <- function(
+  use_buffered_stream = FALSE,
+  buffer_size = 8196,
+  pre_buffer = TRUE,
+  thrift_string_size_limit = 100000000,
+  thrift_container_size_limit = 1000000
+) {
   dataset___ParquetFragmentScanOptions__Make(
-    use_buffered_stream, buffer_size, pre_buffer, thrift_string_size_limit,
+    use_buffered_stream,
+    buffer_size,
+    pre_buffer,
+    thrift_string_size_limit,
     thrift_container_size_limit
   )
 }
@@ -656,7 +664,8 @@ JsonFragmentScanOptions$create <- function(...) {
 #'
 #' @description
 #' A `FileWriteOptions` holds write options specific to a `FileFormat`.
-FileWriteOptions <- R6Class("FileWriteOptions",
+FileWriteOptions <- R6Class(
+  "FileWriteOptions",
   inherit = ArrowObject,
   public = list(
     update = function(column_names, ...) {
